<?php

/**
 * PHPSense Pagination Class
 *
 * PHP tutorials and scripts
 *
 * @package    PHPSense
 * @author     Jatinder Singh Thind
 * @copyright  Copyright (c) 2006, Jatinder Singh Thind
 * @link       http://www.phpsense.com
 * @modified   Tiago Gomes 2010-11-04 (Adaptation to Contenido)
 * @modified   Murat Purc 2014-01-06 - Usage of Contenido-DB class instead of mysql_* functions.
 *                                   - Added $self_url parameter to fix possible issues with Clean-Urls,
 *                                     see http://forum.contenido.org/viewtopic.php?t=33720&p=159435#p159435
 *                                   - Ported class to PHP 5
 */
// ------------------------------------------------------------------------

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

class PS_Pagination {

    // Note: The properties below are public due to downwards compatibility!
    
    public $rows_per_page = 10; //Number of records to display per page
    public $total_rows = 0; //Total number of rows returned by the query
    public $links_per_page = 5; //Number of links to display per page
    public $append = ""; //Parameters to append to pagination links
    public $sql = "";
    public $debug = false;
    /**
     * @var DB_Contenido
     */
    public $conn = null;
    public $page = 1;
    public $max_pages = 0;
    public $offset = 0;

    private $_php_self;

    /**
     * Constructor
     *
     * @param DB_Contenido $connection CONTENIDO DB object instance
     * @param string $sql SQL query to paginate. Example : SELECT * FROM users
     * @param integer $rows_per_page Number of records to display per page. Defaults to 10
     * @param integer $links_per_page Number of links to display per page. Defaults to 5
     * @param string $append Parameters to be appended to pagination links 
     * @param string $self_url Filename of current page, usually "front_content.php"
     */
    public function __construct(DB_Contenido $connection, $sql, $rows_per_page = 10, $links_per_page = 5, $append = "", $self_url = "front_content.php") {
        $this->conn = $connection;
        $this->sql = $sql;
        $this->rows_per_page = (int) $rows_per_page;
        if (intval($links_per_page) > 0) {
            $this->links_per_page = (int) $links_per_page;
        } else {
            $this->links_per_page = 5;
        }
        $this->append = $append;
        $this->_php_self = (!empty($self_url)) ? $self_url : htmlspecialchars($_SERVER['PHP_SELF']);
        if (isset($_GET['page'])) {
            $this->page = intval($_GET['page']);
        }
    }

    /**
     * Executes the SQL query and initializes internal variables
     *
     * @return resource|object|bool
     */
    public function paginate() {
        $all_rs = $this->conn->query($this->sql);
        if (!$all_rs) {
            $this->_debug("SQL query failed. Check your query.<br /><br />Error Returned: " . $this->conn->Error);
            return false;
        }
        $this->total_rows = $this->conn->num_rows();

        //Return false if no rows found
        if ($this->total_rows == 0) {
            $this->_debug("Query returned zero rows.");
            return false;
        }

        //Max number of pages
        $this->max_pages = ceil($this->total_rows / $this->rows_per_page);
        if ($this->links_per_page > $this->max_pages) {
            $this->links_per_page = $this->max_pages;
        }

        //Check the page value just in case someone is trying to input an arbitrary value
        if ($this->page > $this->max_pages || $this->page <= 0) {
            $this->page = 1;
        }

        //Calculate Offset
        $this->offset = $this->rows_per_page * ($this->page - 1);

        //Fetch the required result set
        $rs = $this->conn->query($this->sql . " LIMIT {$this->offset}, {$this->rows_per_page}");
        if (!$rs) {
            $this->_debug("Pagination query failed. Check your query.<br /><br />Error Returned: " . $this->conn->Error);
            return false;
        }
        return $rs;
    }

    /**
     * Display the link to the first page
     *
     * @param string $tag Text string to be displayed as the link. Defaults to 'First'
     * @return string
     */
    public function renderFirst($tag = 'First') {
        if ($this->total_rows == 0) {
            return false;
        }

        if ($this->page == 1) {
            return "$tag ";
        } else {
            return '<a class="pagFirst" href="' . $this->_php_self . '?page=1&' . $this->append . '">' . $tag . '</a> ';
        }
    }

    /**
     * Display the link to the last page
     *
     * @param string $tag Text string to be displayed as the link. Defaults to 'Last'
     * @return string
     */
    public function renderLast($tag = 'Last') {
        if ($this->total_rows == 0) {
            return false;
        }

        if ($this->page == $this->max_pages) {
            return $tag;
        } else {
            return ' <a class="pagLast" href="' . $this->_php_self . '?page=' . $this->max_pages . '&' . $this->append . '">' . $tag . '</a>';
        }
    }

    /**
     * Display the next link
     *
     * @param string $tag Text string to be displayed as the link. Defaults to '>>'
     * @return string
     */
    public function renderNext($tag = '&gt;&gt;') {
        if ($this->total_rows == 0) {
            return false;
        }

        if ($this->page < $this->max_pages) {
            return '<a class="pagNext" href="' . $this->_php_self . '?page=' . ($this->page + 1) . '&' . $this->append . '">' . $tag . '</a> ';
        } else {
            return $tag . ' ';
        }
    }

    /**
     * Display the previous link
     *
     * @param string $tag Text string to be displayed as the link. Defaults to '<<'
     * @return string
     */
    public function renderPrev($tag = '&lt;&lt;') {
        if ($this->total_rows == 0) {
            return false;
        }

        if ($this->page > 1) {
            return ' <a class="pagPrev" href="' . $this->_php_self . '?page=' . ($this->page - 1) . '&' . $this->append . '">' . $tag . '</a>';
        } else {
            return ' ' . $tag;
        }
    }

    /**
     * Display the page links
     *
     * @return string
     */
    public function renderNav($prefix = '<span class="pagLink">', $suffix = '</span>') {
        if ($this->total_rows == 0) {
            return false;
        }

        $batch = ceil($this->page / $this->links_per_page);
        $end = $batch * $this->links_per_page;
        if ($end == $this->page) {
            //$end = $end + $this->links_per_page - 1;
            //$end = $end + ceil($this->links_per_page/2);
        }
        if ($end > $this->max_pages) {
            $end = $this->max_pages;
        }
        $start = $end - $this->links_per_page + 1;
        $links = '';

        for ($i = $start; $i <= $end; $i ++) {
            if ($i == $start) {
                $links .= ' ';
            }
            if ($i == $this->page) {
                $links .= $prefix . '<span class="pagLinkActive">' . $i . '</span>' . $suffix;
            } else {
                $links .= $prefix . '<a class="pagLink" href="' . $this->_php_self . '?page=' . $i . '&' . $this->append . '">' . $i . '</a>' . $suffix;
            }
            $links .= ' ';
        }

        return $links;
    }

    /**
     * Display full pagination navigation
     *
     * @return string
     */
    public function renderFullNav() {
        return $this->renderFirst() . '&nbsp;' . $this->renderPrev() . '&nbsp;' . $this->renderNav() . '&nbsp;' . $this->renderNext() . '&nbsp;' . $this->renderLast();
    }

    /**
     * Set debug mode
     *
     * @param bool $debug Set to TRUE to enable debug messages
     */
    public function setDebug($debug) {
        $this->debug = $debug;
    }

    /**
     * Outputs a debug message if debugging is enabled.
     *
     * @param string $msg The message to output
     * @return void
     */
    private function _debug($msg) {
        if ($this->debug) {
            echo $msg;
        }
    }
}
